package com.tzld.piaoquan.api.service.contentplatform.impl;

import com.alibaba.fastjson.JSONObject;
import com.tzld.piaoquan.api.common.enums.ExceptionEnum;
import com.tzld.piaoquan.api.common.exception.CommonException;
import com.tzld.piaoquan.api.dao.mapper.contentplatform.ContentPlatformGzhPlanMapper;
import com.tzld.piaoquan.api.dao.mapper.contentplatform.ContentPlatformGzhPlanVideoMapper;
import com.tzld.piaoquan.api.dao.mapper.contentplatform.ContentPlatformQwPlanMapper;
import com.tzld.piaoquan.api.dao.mapper.contentplatform.ext.ContentPlatformPlanMapperExt;
import com.tzld.piaoquan.api.model.config.LoginUserContext;
import com.tzld.piaoquan.api.model.param.contentplatform.*;
import com.tzld.piaoquan.api.model.po.contentplatform.*;
import com.tzld.piaoquan.api.model.vo.GhDetailVo;
import com.tzld.piaoquan.api.model.vo.contentplatform.GzhPlanItemVO;
import com.tzld.piaoquan.api.model.vo.contentplatform.QwPlanItemVO;
import com.tzld.piaoquan.api.model.vo.contentplatform.VideoContentItemVO;
import com.tzld.piaoquan.api.remote.AigcApiService;
import com.tzld.piaoquan.api.remote.ManagerApiService;
import com.tzld.piaoquan.api.service.GhDetailService;
import com.tzld.piaoquan.api.service.contentplatform.ContentPlatformCooperateAccountService;
import com.tzld.piaoquan.api.service.contentplatform.ContentPlatformPlanService;
import com.tzld.piaoquan.growth.common.common.enums.GhTypeEnum;
import com.tzld.piaoquan.growth.common.common.enums.StrategyStatusEnum;
import com.tzld.piaoquan.growth.common.model.po.GhDetail;
import com.tzld.piaoquan.growth.common.utils.page.Page;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.collections4.CollectionUtils;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.Objects;
import java.util.stream.Collectors;

@Slf4j
@Service
public class ContentPlatformPlanServiceImpl implements ContentPlatformPlanService {

    @Autowired
    private ContentPlatformGzhPlanMapper gzhPlanMapper;
    @Autowired
    private ContentPlatformPlanMapperExt planMapperExt;
    @Autowired
    private ContentPlatformGzhPlanVideoMapper gzhPlanVideoMapper;
    @Autowired
    private ContentPlatformQwPlanMapper qwPlanMapper;
    @Autowired
    private ContentPlatformCooperateAccountService cooperateAccountService;
    @Autowired
    private GhDetailService ghDetailService;
    @Autowired
    private AigcApiService aigcApiService;
    @Autowired
    private ManagerApiService managerApiService;

    @Override
    public Page<GzhPlanItemVO> gzhPlanList(GzhPlanListParam param) {
        ContentPlatformAccount loginAccount = LoginUserContext.getUser();
        Page<GzhPlanItemVO> result = new Page<>(param.getPageNum(), param.getPageSize());
        int offset = (param.getPageNum() - 1) * param.getPageSize();
        int count = planMapperExt.getGzhPlanCount(param, loginAccount.getId());
        result.setTotalSize(count);
        if (count == 0) {
            return result;
        }
        List<ContentPlatformGzhPlan> planList = planMapperExt.getGzhPlanList(param,
                loginAccount.getId(), offset, param.getPageSize());
        List<GzhPlanItemVO> list = buildGzhPlanItemVOList(planList);
        result.setObjs(list);
        return result;
    }

    private List<GzhPlanItemVO> buildGzhPlanItemVOList(List<ContentPlatformGzhPlan> planList) {
        if (CollectionUtils.isEmpty(planList)) {
            return null;
        }
        List<Long> accountIds = planList.stream().map(ContentPlatformGzhPlan::getAccountId).collect(Collectors.toList());
        List<ContentPlatformGzhAccount> accountList = cooperateAccountService.getAccountListByIds(accountIds);
        Map<Long, String> accountMap = accountList.stream().collect(Collectors.toMap(ContentPlatformGzhAccount::getId,
                ContentPlatformGzhAccount::getName));
        List<Long> planIds = planList.stream().map(ContentPlatformGzhPlan::getId).collect(Collectors.toList());
        List<ContentPlatformGzhPlanVideo> videoList = getGzhPlanVideoList(planIds);
        Map<Long, List<ContentPlatformGzhPlanVideo>> videoMap = videoList.stream()
                .collect(Collectors.groupingBy(ContentPlatformGzhPlanVideo::getPlanId));
        List<GzhPlanItemVO> result = new ArrayList<>();
        for (ContentPlatformGzhPlan gzhPlan : planList) {
            GzhPlanItemVO planItemVO = new GzhPlanItemVO();
            planItemVO.setId(gzhPlan.getId());
            planItemVO.setAccountId(gzhPlan.getAccountId());
            planItemVO.setAccountName(accountMap.get(gzhPlan.getAccountId()));
            planItemVO.setScene(gzhPlan.getScene());
            planItemVO.setPublishStage(gzhPlan.getPublishStage());
            List<ContentPlatformGzhPlanVideo> videoListByPlanId = videoMap.get(gzhPlan.getId());
            if (CollectionUtils.isNotEmpty(videoListByPlanId)) {
                planItemVO.setVideoCount(videoListByPlanId.size());
                List<String> titleList = videoListByPlanId.stream().map(ContentPlatformGzhPlanVideo::getTitle)
                        .collect(Collectors.toList());
                planItemVO.setTitle(titleList);
            }
            planItemVO.setCreateTimestamp(gzhPlan.getCreateTimestamp());
            result.add(planItemVO);
        }
        return result;
    }

    private List<ContentPlatformGzhPlanVideo> getGzhPlanVideoList(List<Long> planIds) {
        ContentPlatformGzhPlanVideoExample example = new ContentPlatformGzhPlanVideoExample();
        example.createCriteria().andPlanIdIn(planIds);
        return gzhPlanVideoMapper.selectByExample(example);
    }

    @Override
    public void gzhPlanSave(GzhPlanSaveParam param) {
        ContentPlatformAccount loginAccount = LoginUserContext.getUser();
        List<ContentPlatformGzhPlan> planExistList = planMapperExt.getGzhPlanExists(param.getAccountId(), param.getScene(), param.getId());
        if (CollectionUtils.isNotEmpty(planExistList)) {
            throw new CommonException(ExceptionEnum.GZH_SCENE_PLAN_EXISTS);
        }
        if (param.getVideoList().size() > 3) {
            throw new CommonException(ExceptionEnum.GZH_PLAN_VIDEO_SIZE_EXCEED);
        }
        ContentPlatformGzhAccount account = cooperateAccountService.getById(param.getAccountId());
        Long now = System.currentTimeMillis();
        ContentPlatformGzhPlan gzhPlan = new ContentPlatformGzhPlan();
        gzhPlan.setAccountId(param.getAccountId());
        gzhPlan.setScene(param.getScene());
        gzhPlan.setPublishStage(param.getPublishStage());
        gzhPlan.setUpdateTimestamp(now);
        // 更新gh_detail
        List<Long> videoIds = param.getVideoList().stream().map(VideoContentItemVO::getVideoId).collect(Collectors.toList());
        updateGhDetail(account, videoIds);
        // 调用aigc创建发布计划
        String externalId = aigcApiService.createPublishPlan(account.getExternalId(), account.getName());
        if (Objects.isNull(param.getId())) {
            gzhPlan.setExternalId(externalId);
            gzhPlan.setCreateAccountId(loginAccount.getId());
            gzhPlan.setCreateTimestamp(now);
            planMapperExt.insertGzhPlanReturnId(gzhPlan);
        } else {
            gzhPlan.setId(param.getId());
            gzhPlanMapper.updateByPrimaryKeySelective(gzhPlan);
        }
        // 保存视频内容
        saveGzhPlanVideo(param, videoIds, gzhPlan.getId(), loginAccount.getId());
    }

    private void saveGzhPlanVideo(GzhPlanSaveParam param, List<Long> videoIds, Long id, Long loginAccountId) {
        List<ContentPlatformGzhPlanVideo> existsVideo = getGzhVideoByPlanId(id);
        List<Long> existsVideoIds = existsVideo.stream().map(ContentPlatformGzhPlanVideo::getVideoId).collect(Collectors.toList());
        Map<Long, ContentPlatformGzhPlanVideo> existsVideoMap = existsVideo.stream()
               .collect(Collectors.toMap(ContentPlatformGzhPlanVideo::getVideoId, item -> item));
        for (ContentPlatformGzhPlanVideo item : existsVideo) {
            if (!videoIds.contains(item.getVideoId())) {
                gzhPlanVideoMapper.deleteByPrimaryKey(item.getId());
            }
        }
        for (VideoContentItemVO vo : param.getVideoList()) {
            if (existsVideoIds.contains(vo.getVideoId())) {
                ContentPlatformGzhPlanVideo item = existsVideoMap.get(vo.getVideoId());
                item.setTitle(vo.getTitle());
                item.setTitleIsUpdate(vo.getTitleIsEdit());
                item.setCover(vo.getCover());
                item.setCoverIsUpdate(vo.getCoverIsEdit());
                gzhPlanVideoMapper.updateByPrimaryKeySelective(item);
            } else {
                ContentPlatformGzhPlanVideo item = new ContentPlatformGzhPlanVideo();
                item.setPlanId(id);
                item.setVideoId(vo.getVideoId());
                item.setTitle(vo.getTitle());
                item.setTitleIsUpdate(vo.getTitleIsEdit());
                item.setCover(vo.getCover());
                item.setCoverIsUpdate(vo.getCoverIsEdit());
                item.setVideo(vo.getVideo());
                item.setCreateAccountId(loginAccountId);
                item.setCreateTimestamp(System.currentTimeMillis());
                gzhPlanVideoMapper.insertSelective(item);
            }
        }
    }

    private List<ContentPlatformGzhPlanVideo> getGzhVideoByPlanId(Long planId) {
        ContentPlatformGzhPlanVideoExample example = new ContentPlatformGzhPlanVideoExample();
        example.createCriteria().andPlanIdEqualTo(planId);
        return gzhPlanVideoMapper.selectByExample(example);
    }

    private void updateGhDetail(ContentPlatformGzhAccount account, List<Long> videoIds) {
        GhDetail ghDetail = ghDetailService.getGhDetailByGhIdType(account.getGhId(), GhTypeEnum.THIRD_PARTY_GH.type);
        GhDetailVo detailVo = new GhDetailVo();
        if (Objects.nonNull(ghDetail)) {
            ghDetail.setStrategyStatus(StrategyStatusEnum.DEFAULT.status);
            ghDetail.setVideoIds(JSONObject.toJSONString(videoIds));
            ghDetail.setAutoreplySendMinigramNum(videoIds.size());
            ghDetail.setCategory1(account.getContentType());
            BeanUtils.copyProperties(ghDetail, detailVo);
            ghDetailService.updateDetail(detailVo);
        } else {
            ContentPlatformAccount loginUser = LoginUserContext.getUser();
            detailVo.setAccountId(account.getGhId());
            detailVo.setAccountName(account.getName());
            detailVo.setType(GhTypeEnum.THIRD_PARTY_GH.type);
            detailVo.setChannel(loginUser.getChannel());
            detailVo.setCategory1(account.getContentType());
            detailVo.setStrategyStatus(StrategyStatusEnum.DEFAULT.status);
            detailVo.setVideoIds(videoIds);
            detailVo.setAutoreplySendMinigramNum(videoIds.size());
            ghDetailService.addGhDetail(detailVo);
        }
    }

    @Override
    public List<ContentPlatformGzhPlan> getGzhPlanListByCooperateAccountId(Long accountId) {
        ContentPlatformGzhPlanExample example = new ContentPlatformGzhPlanExample();
        example.createCriteria().andAccountIdEqualTo(accountId);
        return gzhPlanMapper.selectByExample(example);
    }

    @Override
    public Page<VideoContentItemVO> getVideoContentList(VideoContentListParam param) {
        Page<VideoContentItemVO> result = new Page<>(param.getPageNum(), param.getPageSize());
        int offset = (param.getPageNum() - 1) * param.getPageSize();
        String dt = planMapperExt.getVideoMaxDt();
        int count = planMapperExt.getVideoCount(param, dt);
        result.setTotalSize(count);
        if (count == 0) {
            return result;
        }
        List<ContentPlatformVideo> videoList = planMapperExt.getVideoList(param, dt, offset, param.getPageSize());
        List<VideoContentItemVO> list = buildVideoContentItemVOList(videoList);
        result.setObjs(list);
        return result;
    }

    private List<VideoContentItemVO> buildVideoContentItemVOList(List<ContentPlatformVideo> videoList) {
        if (CollectionUtils.isEmpty(videoList)) {
            return null;
        }
        List<VideoContentItemVO> result = new ArrayList<>();
        for (ContentPlatformVideo video : videoList) {
            VideoContentItemVO item = new VideoContentItemVO();
            item.setVideoId(video.getVideoId());
            item.setTitle(video.getTitle());
            item.setCover(video.getVideo());
            item.setVideo(video.getVideo());
            item.setScore(video.getScore());
            result.add(item);
        }
        return result;
    }

    @Override
    public List<String> getVideoContentCoverFrameList(VideoContentCoverFrameParam param) {
        return managerApiService.getCoverImagePaths(param.getVideoId());
    }

    @Override
    public Page<QwPlanItemVO> qwPlanList(QwPlanListParam param) {
        return null;
    }

    @Override
    public void qwPlanSave(QwPlanSaveParam param) {

    }
}
